=== modified file 'DBUS-API'
--- DBUS-API	2011-11-26 23:08:17 +0000
+++ DBUS-API	2012-01-15 20:27:28 +0000
@@ -93,10 +93,11 @@
    | Interval (a)            | t    | Read/Write | interval            |
    | LastApprovalRequest (g) | s    | Read       | N/A                 |
    | LastCheckedOK (h)       | s    | Read/Write | N/A                 |
-   | LastEnabled (i)         | s    | Read       | N/A                 |
+   | LastCheckerStatus (i)   | n    | Read       | N/A                 |
+   | LastEnabled (j)         | s    | Read       | N/A                 |
    | Name                    | s    | Read       | (Section name)      |
    | ObjectPath              | o    | Read       | N/A                 |
-   | Secret (j)              | ay   | Write      | secret (or secfile) |
+   | Secret (k)              | ay   | Write      | secret (or secfile) |
    | Timeout (a)             | t    | Read/Write | timeout             |
    
    a) Represented as milliseconds.
@@ -124,10 +125,13 @@
       always use an empty string when setting this property, to allow
       for possible future expansion.
    
-   i) The date and time this client was last enabled, as an RFC 3339
+   i) The exit status of the last checker, -1 if it did not exit
+      cleanly, -2 if a checker has not yet returned.
+   
+   j) The date and time this client was last enabled, as an RFC 3339
       string, or an empty string if this has not happened.
    
-   j) A raw byte array, not hexadecimal digits.
+   k) A raw byte array, not hexadecimal digits.
 
 ** Signals
 *** CheckerCompleted(n: Exitcode, x: Waitstatus, s: Command)
@@ -152,13 +156,10 @@
 *** Rejected(s: Reason)
     This client was not given its secret for a specified Reason.
 
-*** NewRequest(s: IP address)
-    A client at IP adress has asked for its password.
-
 * Copyright
 
-    Copyright © 2010-2011 Teddy Hogeborn
-    Copyright © 2010-2011 Björn Påhlsson
+    Copyright © 2010-2012 Teddy Hogeborn
+    Copyright © 2010-2012 Björn Påhlsson
   
 ** License:
    

=== modified file 'Makefile'
--- Makefile	2011-12-21 00:33:12 +0000
+++ Makefile	2012-01-15 21:07:44 +0000
@@ -23,7 +23,7 @@
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
 htmldir=man
-version=1.4.1
+version=1.5.3
 SED=sed
 
 USER=$(firstword $(subst :, ,$(shell getent passwd _mandos || getent passwd nobody || echo 65534)))
@@ -319,6 +319,8 @@
 		> $(MANDIR)/man5/mandos.conf.5.gz
 	gzip --best --to-stdout mandos-clients.conf.5 \
 		> $(MANDIR)/man5/mandos-clients.conf.5.gz
+	gzip --best --to-stdout intro.8mandos \
+		> $(MANDIR)/man8/intro.8mandos.gz
 
 install-client-nokey: all doc
 	install --directory $(PREFIX)/lib/mandos $(CONFDIR)

=== modified file 'NEWS'
--- NEWS	2011-10-15 16:48:03 +0000
+++ NEWS	2012-01-15 21:07:44 +0000
@@ -1,6 +1,42 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
+Version 1.5.3 (2012-01-15)
+* Server
+** Add D-Bus property se.recompile.Client.LastCheckerStatus and use it
+   in mandos-monitor.
+* Client
+** Fix bugs in the example "bridge" network hook.
+
+Version 1.5.2 (2012-01-08)
+* Server
+** Removed D-Bus signal se.recompile.Mandos.NewRequest() added in
+   1.5.0.  It was buggy and was of questionable utility.
+
+Version 1.5.1 (2012-01-01)
+* Server
+** Include intro(8mandos) manual page, missing since migration from
+   README file in version 1.4.0.
+
+Version 1.5.0 (2012-01-01)
+* Client
+** Network hooks.  The Mandos client can now run custom scripts to take
+   up a network interface before the client is run.  Three example
+   scripts are provided: "wireless", "openvpn", and "bridge".
+   To facilitate this, the client now prefers network interfaces which
+   are up (if any) over all other interfaces.
+* Server
+** Persistent state.  Client state is now saved between server
+   restarts.
+** clients.conf file can now contain "enabled" setting for clients.
+** Bug fix: Fix rare crash bug.
+** Bug fix: Send corrent D-Bus type in PropertyChanged for
+   "ApprovalDelay", "ApprovalDuration", "Timeout", and
+   "ExtendedTimeout".
+** mandos-ctl: Bare numbers as arguments are taken to be milliseconds.
+** Bug fix: mandos-ctl --secret option now works.
+** New D-Bus signal: se.recompile.Mandos.NewRequest(s).
+
 Version 1.4.1 (2011-10-15)
 * Server
 ** Make D-Bus properties settable again, and handle checkers

=== modified file 'README'
--- README	2011-10-05 16:00:56 +0000
+++ README	2012-01-01 20:45:53 +0000
@@ -1,7 +1,8 @@
 Please see: http://www.recompile.se/mandos/man/intro.8mandos
 
 This information previously in this file has been moved to the
-intro(8mandos) manual page.  Go to the above URL, or run this command:
+intro(8mandos) manual page.  Go to the above URL, or install the
+Mandos server and run this command:
 
 	man 8mandos intro
 

=== modified file 'TODO'
--- TODO	2011-12-25 00:43:51 +0000
+++ TODO	2012-05-01 20:48:00 +0000
@@ -11,6 +11,7 @@
 ** TODO [#B] Use getaddrinfo(hints=AI_NUMERICHOST) instead of inet_pton()
 ** TODO [#B] Use getnameinfo(serv=NULL, NI_NUMERICHOST) instead of inet_ntop()
 ** TODO [#B] Prefer /run/tmp over /tmp, if it exists
+** TODO [#B] Use in_port_t instead of uint16_t for port numbers.
 
 * splashy
 ** TODO [#B] use scandir(3) instead of readdir(3)
@@ -69,6 +70,19 @@
 ** TODO Allow %%(checker)s as a runtime expansion
 ** TODO Use python-tlslite?
 ** TODO D-Bus AddClient() method on server object
+** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods.
+** TODO Emit [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-properties][org.freedesktop.DBus.Properties.PropertiesChanged]] signal
+   TODO Deprecate se.recompile.Mandos.Client.PropertyChanged - annotate!
+   TODO Can use "invalidates" annotation to also emit on changed secret.
+** TODO Support [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager][org.freedesktop.DBus.ObjectManager]] interface on server object
+   Deprecate methods GetAllClients(), GetAllClientsWithProperties()
+   and signals ClientAdded and ClientRemoved.
+** TODO Save state periodically to recover better from hard shutdowns
+** TODO CheckerCompleted method, deprecate CheckedOK
+** TODO Secret Service API?
+   http://standards.freedesktop.org/secret-service/
+** TODO [[info:python:What's%20New%20in%20Python%202%206][What's New in Python 2.6]]
+*** Start at [[info:python:PEP%203101%20Advanced%20String%20Formatting][PEP 3101 Advanced String Formatting]]
 
 * mandos.xml
 ** Add mandos contact info in manual pages
@@ -87,8 +101,7 @@
 ** Urwid client data displayer
    Better view of client data in the listing
 *** Properties popup
-** Nicer crashes. Stack traces Messes up shell.
-*** Print a nice "We are sorry" message, save stack trace to log.
+** Print a nice "We are sorry" message, save stack trace to log.
 ** Show timeout countdown for approval
 
 * mandos-keygen

=== modified file 'common.ent'
--- common.ent	2011-10-15 16:48:03 +0000
+++ common.ent	2012-01-15 21:07:44 +0000
@@ -1,3 +1,3 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!-- Edited by Makefile -->
-<!ENTITY version "1.4.1">
+<!ENTITY version "1.5.3">

=== modified file 'debian/changelog'
--- debian/changelog	2011-10-15 16:48:03 +0000
+++ debian/changelog	2012-02-21 21:20:12 +0000
@@ -1,3 +1,47 @@
+mandos (1.5.3-1.1) unstable; urgency=low
+
+  * Non-maintainer upload.
+  * Fix "mandos FTBFS on buildds": add build-dependency on locales-all and
+    pass LC_ALL to dh_auto_build to make sure we have and use the en_US.UTF-8
+    locale for manpage creation.
+    (Closes: #656178)
+
+ -- gregor herrmann <gregoa@debian.org>  Tue, 31 Jan 2012 17:56:05 +0100
+
+mandos (1.5.3-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 15 Jan 2012 22:05:54 +0100
+
+mandos (1.5.2-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 08 Jan 2012 11:17:20 +0100
+
+mandos (1.5.1-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 21:53:31 +0100
+
+mandos (1.5.0-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/control (mandos-client/Depends): Added "initramfs-tools".
+  * debian/mandos-client.README.Debian: Corrected mail address and adjust
+                                        wording.
+  * debian/rules (binary-common): Exclude new nework-hooks.d directory
+                                  from dh_fixperms.
+  * debian/mandos-client.README.Debian: Document network hook facility.
+  * debian/mandos-client.docs (network-hooks.d): Added.
+  * debian/mandos.dirs (var/lib/mandos): Added.
+  * debian/mandos.postinst: Fix ownership of /var/lib/mandos.
+  * debian/control (mandos/Depends): Added "python-gnupginterface".
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 05:58:11 +0100
+
 mandos (1.4.1-1) unstable; urgency=low
 
   * New upstream release.

=== modified file 'debian/control'
--- debian/control	2011-12-21 17:09:12 +0000
+++ debian/control	2012-02-21 21:20:12 +0000
@@ -6,7 +6,7 @@
 	   Björn Påhlsson <belorn@recompile.se>
 Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
 	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
-	pkg-config, man
+	pkg-config, man, locales-all
 Standards-Version: 3.9.2
 Vcs-Bzr: http://ftp.recompile.se/pub/mandos/trunk
 Vcs-Browser: http://bzr.recompile.se/loggerhead/mandos/trunk/files

=== modified file 'debian/copyright'
--- debian/copyright	2011-10-10 20:29:58 +0000
+++ debian/copyright	2011-12-31 23:05:34 +0000
@@ -4,8 +4,8 @@
 Source: <http://www.recompile.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008-2011 Teddy Hogeborn
-Copyright: Copyright © 2008-2011 Björn Påhlsson
+Copyright: Copyright © 2008-2012 Teddy Hogeborn
+Copyright: Copyright © 2008-2012 Björn Påhlsson
 License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2011-11-28 22:15:04 +0000
+++ debian/mandos-client.README.Debian	2012-01-01 04:52:07 +0000
@@ -33,11 +33,12 @@
 * Specifying a Client Network Interface
   
   At boot time the network interface to use will by default be
-  automatically detected.  If should result in an incorrect interface,
-  edit the DEVICE setting in the "/etc/initramfs-tools/initramfs.conf"
-  file.  (The default setting is empty, meaning to autodetect the
-  interface.)  *If* the DEVICE setting is changed, it will be
-  necessary to update the initrd image by running the command
+  automatically detected.  If this should result in an incorrect
+  interface, edit the DEVICE setting in the
+  "/etc/initramfs-tools/initramfs.conf" file.  (The default setting is
+  empty, meaning it will autodetect the interface.)  *If* the DEVICE
+  setting is changed, it will be necessary to update the initrd image
+  by running the command
   
 	update-initramfs -k all -u
   

=== modified file 'debian/rules'
--- debian/rules	2011-11-27 14:44:28 +0000
+++ debian/rules	2012-02-21 21:20:12 +0000
@@ -38,12 +38,12 @@
 
 build-arch: build-arch-stamp
 build-arch-stamp: configure-stamp
-	dh_auto_build -- all doc
+	LC_ALL=en_US.utf8 dh_auto_build -- all doc
 	touch $@
 
 build-indep: build-indep-stamp
 build-indep-stamp: configure-stamp
-	dh_auto_build -- doc
+	LC_ALL=en_US.UTF-8 dh_auto_build -- doc
 	touch $@
 
 clean:
@@ -86,7 +86,7 @@
 		--exclude etc/mandos/clients.conf \
 		--exclude etc/mandos/plugins.d \
 		--exclude usr/lib/mandos/plugins.d \
-		--exclude /usr/share/doc/mandos-client/network-hooks.d
+		--exclude usr/share/doc/mandos-client/network-hooks.d
 	dh_installdeb
 	dh_shlibdeps
 	dh_gencontrol

=== modified file 'intro.xml'
--- intro.xml	2011-10-05 16:00:56 +0000
+++ intro.xml	2011-12-31 23:05:34 +0000
@@ -1,7 +1,7 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -31,6 +31,7 @@
     </authorgroup>
     <copyright>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos'
--- mandos	2011-12-23 20:00:52 +0000
+++ mandos	2012-05-01 20:48:00 +0000
@@ -11,8 +11,8 @@
 # "AvahiService" class, and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@
 import types
 import binascii
 import tempfile
+import itertools
 
 import dbus
 import dbus.service
@@ -85,7 +86,7 @@
     except ImportError:
         SO_BINDTODEVICE = None
 
-version = "1.4.1"
+version = "1.5.3"
 stored_state_file = "clients.pickle"
 
 logger = logging.getLogger()
@@ -142,7 +143,8 @@
         self.gnupg.options.meta_interactive = False
         self.gnupg.options.homedir = self.tempdir
         self.gnupg.options.extra_args.extend(['--force-mdc',
-                                              '--quiet'])
+                                              '--quiet',
+                                              '--no-use-agent'])
     
     def __enter__(self):
         return self
@@ -174,7 +176,7 @@
     
     def encrypt(self, data, password):
         self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull) as devnull:
+        with open(os.devnull, "w") as devnull:
             try:
                 proc = self.gnupg.run(['--symmetric'],
                                       create_fhs=['stdin', 'stdout'],
@@ -191,12 +193,12 @@
     
     def decrypt(self, data, password):
         self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull) as devnull:
+        with open(os.devnull, "w") as devnull:
             try:
                 proc = self.gnupg.run(['--decrypt'],
                                       create_fhs=['stdin', 'stdout'],
                                       attach_fhs={'stderr': devnull})
-                with contextlib.closing(proc.handles['stdin'] ) as f:
+                with contextlib.closing(proc.handles['stdin']) as f:
                     f.write(data)
                 with contextlib.closing(proc.handles['stdout']) as f:
                     decrypted_plaintext = f.read()
@@ -275,7 +277,7 @@
         try:
             self.add()
         except dbus.exceptions.DBusException as error:
-            logger.critical("DBusException: %s", error)
+            logger.critical("D-Bus Exception", exc_info=error)
             self.cleanup()
             os._exit(1)
         self.rename_count += 1
@@ -414,14 +416,14 @@
     last_checked_ok: datetime.datetime(); (UTC) or None
     last_checker_status: integer between 0 and 255 reflecting exit
                          status of last checker. -1 reflects crashed
-                         checker, or None.
+                         checker, -2 means no checker completed yet.
     last_enabled: datetime.datetime(); (UTC) or None
     name:       string; from the config file, used in log messages and
                         D-Bus identifiers
     secret:     bytestring; sent verbatim (over TLS) to client
     timeout:    datetime.timedelta(); How long from last_checked_ok
                                       until this client is disabled
-    extended_timeout:   extra long timeout when password has been sent
+    extended_timeout:   extra long timeout when secret has been sent
     runtime_expansions: Allowed attributes for runtime expansion.
     expires:    datetime.datetime(); time (UTC) when a client will be
                 disabled, or None
@@ -459,9 +461,12 @@
 
     @staticmethod
     def config_parser(config):
-        """ Construct a new dict of client settings of this form:
+        """Construct a new dict of client settings of this form:
         { client_name: {setting_name: value, ...}, ...}
-        with exceptions for any special settings as defined above"""
+        with exceptions for any special settings as defined above.
+        NOTE: Must be a pure function. Must return the same result
+        value given the same arguments.
+        """
         settings = {}
         for client_name in config.sections():
             section = dict(config.items(client_name))
@@ -471,7 +476,8 @@
             # Reformat values from string types to Python types
             client["approved_by_default"] = config.getboolean(
                 client_name, "approved_by_default")
-            client["enabled"] = config.getboolean(client_name, "enabled")
+            client["enabled"] = config.getboolean(client_name,
+                                                  "enabled")
             
             client["fingerprint"] = (section["fingerprint"].upper()
                                      .replace(" ", ""))
@@ -496,15 +502,8 @@
             client["checker_command"] = section["checker"]
             client["last_approval_request"] = None
             client["last_checked_ok"] = None
-            client["last_checker_status"] = None
-            if client["enabled"]:
-                client["last_enabled"] = datetime.datetime.utcnow()
-                client["expires"] = (datetime.datetime.utcnow()
-                                     + client["timeout"])
-            else:
-                client["last_enabled"] = None
-                client["expires"] = None
-
+            client["last_checker_status"] = -2
+        
         return settings
         
         
@@ -517,12 +516,23 @@
         for setting, value in settings.iteritems():
             setattr(self, setting, value)
         
+        if self.enabled:
+            if not hasattr(self, "last_enabled"):
+                self.last_enabled = datetime.datetime.utcnow()
+            if not hasattr(self, "expires"):
+                self.expires = (datetime.datetime.utcnow()
+                                + self.timeout)
+        else:
+            self.last_enabled = None
+            self.expires = None
+       
         logger.debug("Creating client %r", self.name)
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
         logger.debug("  Fingerprint: %s", self.fingerprint)
-        self.created = settings.get("created", datetime.datetime.utcnow())
+        self.created = settings.get("created",
+                                    datetime.datetime.utcnow())
 
         # attributes specific for this server instance
         self.checker = None
@@ -617,15 +627,16 @@
             logger.warning("Checker for %(name)s crashed?",
                            vars(self))
     
-    def checked_ok(self, timeout=None):
-        """Bump up the timeout for this client.
-        
-        This should only be called when the client has been seen,
-        alive and well.
-        """
+    def checked_ok(self):
+        """Assert that the client has been seen, alive and well."""
+        self.last_checked_ok = datetime.datetime.utcnow()
+        self.last_checker_status = 0
+        self.bump_timeout()
+    
+    def bump_timeout(self, timeout=None):
+        """Bump up the timeout for this client."""
         if timeout is None:
             timeout = self.timeout
-        self.last_checked_ok = datetime.datetime.utcnow()
         if self.disable_initiator_tag is not None:
             gobject.source_remove(self.disable_initiator_tag)
         if getattr(self, "enabled", False):
@@ -682,8 +693,8 @@
                 try:
                     command = self.checker_command % escaped_attrs
                 except TypeError as error:
-                    logger.error('Could not format string "%s":'
-                                 ' %s', self.checker_command, error)
+                    logger.error('Could not format string "%s"',
+                                 self.checker_command, exc_info=error)
                     return True # Try again later
             self.current_checker_command = command
             try:
@@ -707,8 +718,8 @@
                     gobject.source_remove(self.checker_callback_tag)
                     self.checker_callback(pid, status, command)
             except OSError as error:
-                logger.error("Failed to start subprocess: %s",
-                             error)
+                logger.error("Failed to start subprocess",
+                             exc_info=error)
         # Re-run this periodically if run by gobject.timeout_add
         return True
     
@@ -721,10 +732,10 @@
             return
         logger.debug("Stopping checker for %(name)s", vars(self))
         try:
-            os.kill(self.checker.pid, signal.SIGTERM)
+            self.checker.terminate()
             #time.sleep(0.5)
             #if self.checker.poll() is None:
-            #    os.kill(self.checker.pid, signal.SIGKILL)
+            #    self.checker.kill()
         except OSError as error:
             if error.errno != errno.ESRCH: # No such process
                 raise
@@ -761,6 +772,43 @@
     return decorator
 
 
+def dbus_interface_annotations(dbus_interface):
+    """Decorator for marking functions returning interface annotations.
+    
+    Usage:
+    
+    @dbus_interface_annotations("org.example.Interface")
+    def _foo(self):  # Function name does not matter
+        return {"org.freedesktop.DBus.Deprecated": "true",
+                "org.freedesktop.DBus.Property.EmitsChangedSignal":
+                    "false"}
+    """
+    def decorator(func):
+        func._dbus_is_interface = True
+        func._dbus_interface = dbus_interface
+        func._dbus_name = dbus_interface
+        return func
+    return decorator
+
+
+def dbus_annotations(annotations):
+    """Decorator to annotate D-Bus methods, signals or properties
+    Usage:
+    
+    @dbus_service_property("org.example.Interface", signature="b",
+                           access="r")
+    @dbus_annotations({{"org.freedesktop.DBus.Deprecated": "true",
+                        "org.freedesktop.DBus.Property."
+                        "EmitsChangedSignal": "false"})
+    def Property_dbus_property(self):
+        return dbus.Boolean(False)
+    """
+    def decorator(func):
+        func._dbus_annotations = annotations
+        return func
+    return decorator
+
+
 class DBusPropertyException(dbus.exceptions.DBusException):
     """A base class for D-Bus property-related exceptions
     """
@@ -789,16 +837,25 @@
     """
     
     @staticmethod
-    def _is_dbus_property(obj):
-        return getattr(obj, "_dbus_is_property", False)
+    def _is_dbus_thing(thing):
+        """Returns a function testing if an attribute is a D-Bus thing
+        
+        If called like _is_dbus_thing("method") it returns a function
+        suitable for use as predicate to inspect.getmembers().
+        """
+        return lambda obj: getattr(obj, "_dbus_is_{0}".format(thing),
+                                   False)
     
-    def _get_all_dbus_properties(self):
+    def _get_all_dbus_things(self, thing):
         """Returns a generator of (name, attribute) pairs
         """
-        return ((prop.__get__(self)._dbus_name, prop.__get__(self))
+        return ((getattr(athing.__get__(self), "_dbus_name",
+                         name),
+                 athing.__get__(self))
                 for cls in self.__class__.__mro__
-                for name, prop in
-                inspect.getmembers(cls, self._is_dbus_property))
+                for name, athing in
+                inspect.getmembers(cls,
+                                   self._is_dbus_thing(thing)))
     
     def _get_dbus_property(self, interface_name, property_name):
         """Returns a bound method if one exists which is a D-Bus
@@ -806,7 +863,8 @@
         """
         for cls in  self.__class__.__mro__:
             for name, value in (inspect.getmembers
-                                (cls, self._is_dbus_property)):
+                                (cls,
+                                 self._is_dbus_thing("property"))):
                 if (value._dbus_name == property_name
                     and value._dbus_interface == interface_name):
                     return value.__get__(self)
@@ -841,8 +899,8 @@
             # signatures other than "ay".
             if prop._dbus_signature != "ay":
                 raise ValueError
-            value = dbus.ByteArray(''.join(unichr(byte)
-                                           for byte in value))
+            value = dbus.ByteArray(b''.join(chr(byte)
+                                            for byte in value))
         prop(value)
     
     @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature="s",
@@ -854,7 +912,7 @@
         Note: Will not include properties with access="write".
         """
         properties = {}
-        for name, prop in self._get_all_dbus_properties():
+        for name, prop in self._get_all_dbus_things("property"):
             if (interface_name
                 and interface_name != prop._dbus_interface):
                 # Interface non-empty but did not match
@@ -875,7 +933,9 @@
                          path_keyword='object_path',
                          connection_keyword='connection')
     def Introspect(self, object_path, connection):
-        """Standard D-Bus method, overloaded to insert property tags.
+        """Overloading of standard D-Bus method.
+        
+        Inserts property tags and interface annotation tags.
         """
         xmlstring = dbus.service.Object.Introspect(self, object_path,
                                                    connection)
@@ -888,12 +948,44 @@
                 e.setAttribute("access", prop._dbus_access)
                 return e
             for if_tag in document.getElementsByTagName("interface"):
+                # Add property tags
                 for tag in (make_tag(document, name, prop)
                             for name, prop
-                            in self._get_all_dbus_properties()
+                            in self._get_all_dbus_things("property")
                             if prop._dbus_interface
                             == if_tag.getAttribute("name")):
                     if_tag.appendChild(tag)
+                # Add annotation tags
+                for typ in ("method", "signal", "property"):
+                    for tag in if_tag.getElementsByTagName(typ):
+                        annots = dict()
+                        for name, prop in (self.
+                                           _get_all_dbus_things(typ)):
+                            if (name == tag.getAttribute("name")
+                                and prop._dbus_interface
+                                == if_tag.getAttribute("name")):
+                                annots.update(getattr
+                                              (prop,
+                                               "_dbus_annotations",
+                                               {}))
+                        for name, value in annots.iteritems():
+                            ann_tag = document.createElement(
+                                "annotation")
+                            ann_tag.setAttribute("name", name)
+                            ann_tag.setAttribute("value", value)
+                            tag.appendChild(ann_tag)
+                # Add interface annotation tags
+                for annotation, value in dict(
+                    itertools.chain(
+                        *(annotations().iteritems()
+                          for name, annotations in
+                          self._get_all_dbus_things("interface")
+                          if name == if_tag.getAttribute("name")
+                          ))).iteritems():
+                    ann_tag = document.createElement("annotation")
+                    ann_tag.setAttribute("name", annotation)
+                    ann_tag.setAttribute("value", value)
+                    if_tag.appendChild(ann_tag)
                 # Add the names to the return values for the
                 # "org.freedesktop.DBus.Properties" methods
                 if (if_tag.getAttribute("name")
@@ -914,7 +1006,7 @@
         except (AttributeError, xml.dom.DOMException,
                 xml.parsers.expat.ExpatError) as error:
             logger.error("Failed to override Introspection method",
-                         error)
+                         exc_info=error)
         return xmlstring
 
 
@@ -934,6 +1026,7 @@
     def __new__(mcs, name, bases, attr):
         # Go through all the base classes which could have D-Bus
         # methods, signals, or properties in them
+        old_interface_names = []
         for base in (b for b in bases
                      if issubclass(b, dbus.service.Object)):
             # Go though all attributes of the base class
@@ -949,6 +1042,8 @@
                 alt_interface = (attribute._dbus_interface
                                  .replace("se.recompile.Mandos",
                                           "se.bsnet.fukt.Mandos"))
+                if alt_interface != attribute._dbus_interface:
+                    old_interface_names.append(alt_interface)
                 # Is this a D-Bus signal?
                 if getattr(attribute, "_dbus_is_signal", False):
                     # Extract the original non-method function by
@@ -969,6 +1064,12 @@
                                 nonmethod_func.func_name,
                                 nonmethod_func.func_defaults,
                                 nonmethod_func.func_closure)))
+                    # Copy annotations, if any
+                    try:
+                        new_function._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
                     # Define a creator of a function to call both the
                     # old and new functions, so both the old and new
                     # signals gets sent when the function is called
@@ -1002,6 +1103,12 @@
                                         attribute.func_name,
                                         attribute.func_defaults,
                                         attribute.func_closure)))
+                    # Copy annotations, if any
+                    try:
+                        attr[attrname]._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
                 # Is this a D-Bus property?
                 elif getattr(attribute, "_dbus_is_property", False):
                     # Create a new, but exactly alike, function
@@ -1021,6 +1128,38 @@
                                         attribute.func_name,
                                         attribute.func_defaults,
                                         attribute.func_closure)))
+                    # Copy annotations, if any
+                    try:
+                        attr[attrname]._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
+                # Is this a D-Bus interface?
+                elif getattr(attribute, "_dbus_is_interface", False):
+                    # Create a new, but exactly alike, function
+                    # object.  Decorate it to be a new D-Bus interface
+                    # with the alternate D-Bus interface name.  Add it
+                    # to the class.
+                    attr[attrname] = (dbus_interface_annotations
+                                      (alt_interface)
+                                      (types.FunctionType
+                                       (attribute.func_code,
+                                        attribute.func_globals,
+                                        attribute.func_name,
+                                        attribute.func_defaults,
+                                        attribute.func_closure)))
+        # Deprecate all old interfaces
+        basename="_AlternateDBusNamesMetaclass_interface_annotation{0}"
+        for old_interface_name in old_interface_names:
+            @dbus_interface_annotations(old_interface_name)
+            def func(self):
+                return { "org.freedesktop.DBus.Deprecated": "true" }
+            # Find an unused name
+            for aname in (basename.format(i) for i in
+                          itertools.count()):
+                if aname not in attr:
+                    attr[aname] = func
+                    break
         return type.__new__(mcs, name, bases, attr)
 
 
@@ -1040,9 +1179,6 @@
     def __init__(self, bus = None, *args, **kwargs):
         self.bus = bus
         Client.__init__(self, *args, **kwargs)
-        self._approvals_pending = 0
-        
-        self._approvals_pending = 0
         # Only now, when this client is initialized, can it show up on
         # the D-Bus
         client_object_name = unicode(self.name).translate(
@@ -1094,6 +1230,8 @@
                                        checker is not None)
     last_checked_ok = notifychangeproperty(datetime_to_dbus,
                                            "LastCheckedOK")
+    last_checker_status = notifychangeproperty(dbus.Int16,
+                                               "LastCheckerStatus")
     last_approval_request = notifychangeproperty(
         datetime_to_dbus, "LastApprovalRequest")
     approved_by_default = notifychangeproperty(dbus.Boolean,
@@ -1177,6 +1315,13 @@
     ## D-Bus methods, signals & properties
     _interface = "se.recompile.Mandos.Client"
     
+    ## Interfaces
+    
+    @dbus_interface_annotations(_interface)
+    def _foo(self):
+        return { "org.freedesktop.DBus.Property.EmitsChangedSignal":
+                     "false"}
+    
     ## Signals
     
     # CheckerCompleted - signal
@@ -1218,14 +1363,6 @@
         "D-Bus signal"
         return self.need_approval()
     
-    # NeRwequest - signal
-    @dbus.service.signal(_interface, signature="s")
-    def NewRequest(self, ip):
-        """D-Bus signal
-        Is sent after a client request a password.
-        """
-        pass
-    
     ## Methods
     
     # Approve - method
@@ -1341,6 +1478,12 @@
             return
         return datetime_to_dbus(self.last_checked_ok)
     
+    # LastCheckerStatus - property
+    @dbus_service_property(_interface, signature="n",
+                           access="read")
+    def LastCheckerStatus_dbus_property(self):
+        return dbus.Int16(self.last_checker_status)
+    
     # Expires - property
     @dbus_service_property(_interface, signature="s", access="read")
     def Expires_dbus_property(self):
@@ -1358,26 +1501,25 @@
         if value is None:       # get
             return dbus.UInt64(self.timeout_milliseconds())
         self.timeout = datetime.timedelta(0, 0, 0, value)
-        if getattr(self, "disable_initiator_tag", None) is None:
-            return
         # Reschedule timeout
-        gobject.source_remove(self.disable_initiator_tag)
-        self.disable_initiator_tag = None
-        self.expires = None
-        time_to_die = timedelta_to_milliseconds((self
-                                                 .last_checked_ok
-                                                 + self.timeout)
-                                                - datetime.datetime
-                                                .utcnow())
-        if time_to_die <= 0:
-            # The timeout has passed
-            self.disable()
-        else:
-            self.expires = (datetime.datetime.utcnow()
-                            + datetime.timedelta(milliseconds =
-                                                 time_to_die))
-            self.disable_initiator_tag = (gobject.timeout_add
-                                          (time_to_die, self.disable))
+        if self.enabled:
+            now = datetime.datetime.utcnow()
+            time_to_die = timedelta_to_milliseconds(
+                (self.last_checked_ok + self.timeout) - now)
+            if time_to_die <= 0:
+                # The timeout has passed
+                self.disable()
+            else:
+                self.expires = (now +
+                                datetime.timedelta(milliseconds =
+                                                   time_to_die))
+                if (getattr(self, "disable_initiator_tag", None)
+                    is None):
+                    return
+                gobject.source_remove(self.disable_initiator_tag)
+                self.disable_initiator_tag = (gobject.timeout_add
+                                              (time_to_die,
+                                               self.disable))
     
     # ExtendedTimeout - property
     @dbus_service_property(_interface, signature="t",
@@ -1539,10 +1681,6 @@
                 except KeyError:
                     return
                 
-                if self.server.use_dbus:
-                    # Emit D-Bus signal
-                    client.NewRequest(str(self.client_address))
-                
                 if client.approval_delay:
                     delay = client.approval_delay
                     client.approvals_pending += 1
@@ -1612,7 +1750,7 @@
                 
                 logger.info("Sending secret to %s", client.name)
                 # bump the timeout using extended_timeout
-                client.checked_ok(client.extended_timeout)
+                client.bump_timeout(client.extended_timeout)
                 if self.server.use_dbus:
                     # Emit D-Bus signal
                     client.GotSecret()
@@ -1961,11 +2099,11 @@
         sys.exit()
     if not noclose:
         # Close all standard open file descriptors
-        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
         if not stat.S_ISCHR(os.fstat(null).st_mode):
             raise OSError(errno.ENODEV,
                           "%s not a character device"
-                          % os.path.devnull)
+                          % os.devnull)
         os.dup2(null, sys.stdin.fileno())
         os.dup2(null, sys.stdout.fileno())
         os.dup2(null, sys.stderr.fileno())
@@ -2094,7 +2232,8 @@
                                 % server_settings["servicename"]))
     
     # Parse config file with clients
-    client_config = configparser.SafeConfigParser(Client.client_defaults)
+    client_config = configparser.SafeConfigParser(Client
+                                                  .client_defaults)
     client_config.read(os.path.join(server_settings["configdir"],
                                     "clients.conf"))
     
@@ -2117,20 +2256,16 @@
         except IOError:
             logger.error("Could not open file %r", pidfilename)
     
-    try:
-        uid = pwd.getpwnam("_mandos").pw_uid
-        gid = pwd.getpwnam("_mandos").pw_gid
-    except KeyError:
+    for name in ("_mandos", "mandos", "nobody"):
         try:
-            uid = pwd.getpwnam("mandos").pw_uid
-            gid = pwd.getpwnam("mandos").pw_gid
+            uid = pwd.getpwnam(name).pw_uid
+            gid = pwd.getpwnam(name).pw_gid
+            break
         except KeyError:
-            try:
-                uid = pwd.getpwnam("nobody").pw_uid
-                gid = pwd.getpwnam("nobody").pw_gid
-            except KeyError:
-                uid = 65534
-                gid = 65534
+            continue
+    else:
+        uid = 65534
+        gid = 65534
     try:
         os.setgid(gid)
         os.setuid(uid)
@@ -2153,7 +2288,7 @@
          .gnutls_global_set_log_function(debug_gnutls))
         
         # Redirect stdin so all checkers get /dev/null
-        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
         os.dup2(null, sys.stdin.fileno())
         if null > 2:
             os.close(null)
@@ -2167,7 +2302,7 @@
     
     global main_loop
     # From the Avahi example code
-    DBusGMainLoop(set_as_default=True )
+    DBusGMainLoop(set_as_default=True)
     main_loop = gobject.MainLoop()
     bus = dbus.SystemBus()
     # End of Avahi example code
@@ -2241,28 +2376,31 @@
             
             # Clients who has passed its expire date can still be
             # enabled if its last checker was successful.  Clients
-            # whose checker failed before we stored its state is
-            # assumed to have failed all checkers during downtime.
+            # whose checker succeeded before we stored its state is
+            # assumed to have successfully run all checkers during
+            # downtime.
             if client["enabled"]:
                 if datetime.datetime.utcnow() >= client["expires"]:
                     if not client["last_checked_ok"]:
                         logger.warning(
                             "disabling client {0} - Client never "
-                            "performed a successfull checker"
-                            .format(client["name"]))
+                            "performed a successful checker"
+                            .format(client_name))
                         client["enabled"] = False
                     elif client["last_checker_status"] != 0:
                         logger.warning(
                             "disabling client {0} - Client "
                             "last checker failed with error code {1}"
-                            .format(client["name"],
+                            .format(client_name,
                                     client["last_checker_status"]))
                         client["enabled"] = False
                     else:
                         client["expires"] = (datetime.datetime
                                              .utcnow()
                                              + client["timeout"])
-                    
+                        logger.debug("Last checker succeeded,"
+                                     " keeping {0} enabled"
+                                     .format(client_name))
             try:
                 client["secret"] = (
                     pgp.decrypt(client["encrypted_secret"],
@@ -2277,12 +2415,14 @@
 
     
     # Add/remove clients based on new changes made to config
-    for client_name in set(old_client_settings) - set(client_settings):
+    for client_name in (set(old_client_settings)
+                        - set(client_settings)):
         del clients_data[client_name]
-    for client_name in set(client_settings) - set(old_client_settings):
+    for client_name in (set(client_settings)
+                        - set(old_client_settings)):
         clients_data[client_name] = client_settings[client_name]
 
-    # Create clients all clients
+    # Create all client objects
     for client_name, client in clients_data.iteritems():
         tcp_server.clients[client_name] = client_class(
             name = client_name, settings = client)
@@ -2309,12 +2449,18 @@
     signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
     
     if use_dbus:
-        class MandosDBusService(dbus.service.Object):
+        class MandosDBusService(DBusObjectWithProperties):
             """A D-Bus proxy object"""
             def __init__(self):
                 dbus.service.Object.__init__(self, bus, "/")
             _interface = "se.recompile.Mandos"
             
+            @dbus_interface_annotations(_interface)
+            def _foo(self):
+                return { "org.freedesktop.DBus.Property"
+                         ".EmitsChangedSignal":
+                             "false"}
+            
             @dbus.service.signal(_interface, signature="o")
             def ClientAdded(self, objpath):
                 "D-Bus signal"
@@ -2466,7 +2612,7 @@
         try:
             service.activate()
         except dbus.exceptions.DBusException as error:
-            logger.critical("DBusException: %s", error)
+            logger.critical("D-Bus Exception", exc_info=error)
             cleanup()
             sys.exit(1)
         # End of Avahi example code
@@ -2479,7 +2625,7 @@
         logger.debug("Starting main loop")
         main_loop.run()
     except AvahiError as error:
-        logger.critical("AvahiError: %s", error)
+        logger.critical("Avahi Error", exc_info=error)
         cleanup()
         sys.exit(1)
     except KeyboardInterrupt:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2011-11-26 23:08:17 +0000
+++ mandos-clients.conf.xml	2012-01-01 04:02:00 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,6 +36,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -65,8 +66,13 @@
       ><refentrytitle>mandos</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>, read by it at startup.
       The file needs to list all clients that should be able to use
-      the service.  All clients listed will be regarded as enabled,
-      even if a client was disabled in a previous run of the server.
+      the service.  The settings in this file can be overridden by
+      runtime changes to the server, which it saves across restarts.
+      (See the section called <quote>PERSISTENT STATE</quote> in
+      <citerefentry><refentrytitle>mandos</refentrytitle><manvolnum
+      >8</manvolnum></citerefentry>.)  However, any <emphasis
+      >changes</emphasis> to this file (including adding and removing
+      clients) will, at startup, override changes done during runtime.
     </para>
     <para>
       The format starts with a <literal>[<replaceable>section

=== modified file 'mandos-ctl'
--- mandos-ctl	2011-12-21 17:17:32 +0000
+++ mandos-ctl	2012-01-15 21:07:44 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@
 server_path = "/"
 server_interface = domain + ".Mandos"
 client_interface = domain + ".Mandos.Client"
-version = "1.4.1"
+version = "1.5.3"
 
 def timedelta_to_milliseconds(td):
     """Convert a datetime.timedelta object to milliseconds"""
@@ -331,8 +331,7 @@
                            dbus_interface=dbus.PROPERTIES_IFACE)
             if options.secret is not None:
                 client.Set(client_interface, "Secret",
-                           dbus.ByteArray(open(options.secret,
-                                               "rb").read()),
+                           dbus.ByteArray(options.secret.read()),
                            dbus_interface=dbus.PROPERTIES_IFACE)
             if options.approved_by_default is not None:
                 client.Set(client_interface, "ApprovedByDefault",

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2011-10-05 16:00:56 +0000
+++ mandos-ctl.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-keygen'
--- mandos-keygen	2011-10-15 16:48:03 +0000
+++ mandos-keygen	2012-01-15 21:07:44 +0000
@@ -2,8 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 # Contact the authors at <mandos@recompile.se>.
 # 
 
-VERSION="1.4.1"
+VERSION="1.5.3"
 
 KEYDIR="/etc/keys/mandos"
 KEYTYPE=DSA

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2011-11-12 18:14:55 +0000
+++ mandos-keygen.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-monitor'
--- mandos-monitor	2011-10-15 16:48:03 +0000
+++ mandos-monitor	2012-01-15 21:07:44 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2009-2011 Teddy Hogeborn
-# Copyright © 2009-2011 Björn Påhlsson
+# Copyright © 2009-2012 Teddy Hogeborn
+# Copyright © 2009-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@
 domain = 'se.recompile'
 server_interface = domain + '.Mandos'
 client_interface = domain + '.Mandos.Client'
-version = "1.4.1"
+version = "1.5.3"
 
 # Always run in monochrome mode
 urwid.curses_display.curses.has_colors = lambda : False
@@ -131,7 +131,6 @@
         
         self._update_timer_callback_tag = None
         self._update_timer_callback_lock = 0
-        self.last_checker_failed = False
         
         # The widget shown normally
         self._text_widget = urwid.Text("")
@@ -145,17 +144,8 @@
         
         last_checked_ok = isoformat_to_datetime(self.properties
                                                 ["LastCheckedOK"])
-        if last_checked_ok is None:
-            self.last_checker_failed = True
-        else:
-            self.last_checker_failed = ((datetime.datetime.utcnow()
-                                         - last_checked_ok)
-                                        > datetime.timedelta
-                                        (milliseconds=
-                                         self.properties
-                                         ["Interval"]))
         
-        if self.last_checker_failed:
+        if self.properties ["LastCheckerStatus"] != 0:
             self.using_timer(True)
         
         if self.need_approval:
@@ -189,7 +179,12 @@
                                                          value)
         if property == "ApprovalPending":
             using_timer(bool(value))
-        
+        if property == "LastCheckerStatus":
+            using_timer(value != 0)
+            #self.logger('Checker for client %s (command "%s")'
+            #            ' was successful'
+            #            % (self.properties["Name"], command))
+    
     def using_timer(self, flag):
         """Call this method with True or False when timer should be
         activated or deactivated.
@@ -210,18 +205,9 @@
     
     def checker_completed(self, exitstatus, condition, command):
         if exitstatus == 0:
-            if self.last_checker_failed:
-                self.last_checker_failed = False
-                self.using_timer(False)
-            #self.logger('Checker for client %s (command "%s")'
-            #            ' was successful'
-            #            % (self.properties["Name"], command))
             self.update()
             return
         # Checker failed
-        if not self.last_checker_failed:
-            self.last_checker_failed = True
-            self.using_timer(True)
         if os.WIFEXITED(condition):
             self.logger('Checker for client %s (command "%s")'
                         ' failed with exit code %s'
@@ -242,12 +228,13 @@
         self.update()
     
     def checker_started(self, command):
+        """Server signals that a checker started. This could be useful
+           to log in the future. """
         #self.logger('Client %s started checker "%s"'
         #            % (self.properties["Name"], unicode(command)))
         pass
     
     def got_secret(self):
-        self.last_checker_failed = False
         self.logger('Client %s received its secret'
                     % self.properties["Name"])
     
@@ -314,7 +301,7 @@
             else:
                 message = "Denial in %s. (a)pprove?"
             message = message % unicode(timer).rsplit(".", 1)[0]
-        elif self.last_checker_failed:
+        elif self.properties["LastCheckerStatus"] != 0:
             # When checker has failed, print a timer until client expires
             expires = self.properties["Expires"]
             if expires == "":
@@ -497,48 +484,6 @@
         
         self.busname = domain + '.Mandos'
         self.main_loop = gobject.MainLoop()
-        self.bus = dbus.SystemBus()
-        mandos_dbus_objc = self.bus.get_object(
-            self.busname, "/", follow_name_owner_changes=True)
-        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
-                                          dbus_interface
-                                          = server_interface)
-        try:
-            mandos_clients = (self.mandos_serv
-                              .GetAllClientsWithProperties())
-        except dbus.exceptions.DBusException:
-            mandos_clients = dbus.Dictionary()
-        
-        (self.mandos_serv
-         .connect_to_signal("ClientRemoved",
-                            self.find_and_remove_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientAdded",
-                            self.add_new_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientNotFound",
-                            self.client_not_found,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        for path, client in mandos_clients.iteritems():
-            client_proxy_object = self.bus.get_object(self.busname,
-                                                      path)
-            self.add_client(MandosClientWidget(server_proxy_object
-                                               =self.mandos_serv,
-                                               proxy_object
-                                               =client_proxy_object,
-                                               properties=client,
-                                               update_hook
-                                               =self.refresh,
-                                               delete_hook
-                                               =self.remove_client,
-                                               logger
-                                               =self.log_message),
-                            path=path)
     
     def client_not_found(self, fingerprint, address):
         self.log_message(("Client with address %s and fingerprint %s"
@@ -559,7 +504,6 @@
                                                      self.divider)))
         if self.log_visible:
             self.uilist.append(self.logbox)
-            pass
         self.topwidget = urwid.Pile(self.uilist)
     
     def log_message(self, message):
@@ -649,6 +593,49 @@
     
     def run(self):
         """Start the main loop and exit when it's done."""
+        self.bus = dbus.SystemBus()
+        mandos_dbus_objc = self.bus.get_object(
+            self.busname, "/", follow_name_owner_changes=True)
+        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
+                                          dbus_interface
+                                          = server_interface)
+        try:
+            mandos_clients = (self.mandos_serv
+                              .GetAllClientsWithProperties())
+        except dbus.exceptions.DBusException:
+            mandos_clients = dbus.Dictionary()
+        
+        (self.mandos_serv
+         .connect_to_signal("ClientRemoved",
+                            self.find_and_remove_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientAdded",
+                            self.add_new_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientNotFound",
+                            self.client_not_found,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        for path, client in mandos_clients.iteritems():
+            client_proxy_object = self.bus.get_object(self.busname,
+                                                      path)
+            self.add_client(MandosClientWidget(server_proxy_object
+                                               =self.mandos_serv,
+                                               proxy_object
+                                               =client_proxy_object,
+                                               properties=client,
+                                               update_hook
+                                               =self.refresh,
+                                               delete_hook
+                                               =self.remove_client,
+                                               logger
+                                               =self.log_message),
+                            path=path)
+
         self.refresh()
         self._input_callback_tag = (gobject.io_add_watch
                                     (sys.stdin.fileno(),

=== modified file 'mandos-monitor.xml'
--- mandos-monitor.xml	2011-10-05 16:00:56 +0000
+++ mandos-monitor.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-monitor">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2011-11-26 22:22:20 +0000
+++ mandos-options.xml	2012-01-01 04:02:00 +0000
@@ -87,8 +87,8 @@
   </para>
   
   <para id="restore">
-    This option controls whether the server will restore any state from
-    last time it ran. Default is to try restore last state.
+    This option controls whether the server will restore its state
+    from the last time it ran.  Default is to restore last state.
   </para>
   
   <para id="statedir">

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2011-11-26 22:22:20 +0000
+++ mandos.conf.xml	2011-12-31 23:05:34 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos.lsm'
--- mandos.lsm	2011-10-15 16:48:03 +0000
+++ mandos.lsm	2012-01-15 21:07:44 +0000
@@ -1,7 +1,7 @@
 Begin4
 Title:		Mandos
-Version:	1.4.1
-Entered-date:	2011-10-15
+Version:	1.5.3
+Entered-date:	2012-01-15
 Description:	The Mandos system allows computers to have encrypted
 root file systems and at the same time be capable of remote and/or
 unattended reboots.
@@ -12,9 +12,9 @@
 Maintained-by:	teddy@recompile.se (Teddy Hogeborn),
 		belorn@recompile.se (Björn Påhlsson)
 Primary-site: http://www.recompile.se/mandos
-	      138K mandos_1.4.1.orig.tar.gz
+	      147K mandos_1.5.3.orig.tar.gz
 Alternate-site: ftp://ftp.recompile.se/pub/mandos
-		138K mandos_1.4.1.orig.tar.gz
+		147K mandos_1.5.3.orig.tar.gz
 Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.6, and
 various other libraries.  While made for Debian GNU/Linux, it is
 probably portable to other distributions, but not other Unixes.

=== modified file 'mandos.xml'
--- mandos.xml	2011-12-21 00:29:33 +0000
+++ mandos.xml	2012-01-15 21:01:13 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-15">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -285,6 +286,9 @@
 	<term><option>--no-restore</option></term>
 	<listitem>
 	  <xi:include href="mandos-options.xml" xpointer="restore"/>
+	  <para>
+	    See also <xref linkend="persistent_state"/>.
+	  </para>
 	</listitem>
       </varlistentry>
       
@@ -377,9 +381,7 @@
       extended timeout, checker program, and interval between checks
       can be configured both globally and per client; see
       <citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
-      <manvolnum>5</manvolnum></citerefentry>.  A client successfully
-      receiving its password will also be treated as a successful
-      checker run.
+      <manvolnum>5</manvolnum></citerefentry>.
     </para>
   </refsect1>
   
@@ -413,6 +415,18 @@
     </para>
   </refsect1>
   
+  <refsect1 id="persistent_state">
+    <title>PERSISTENT STATE</title>
+    <para>
+      Client settings, initially read from
+      <filename>clients.conf</filename>, are persistent across
+      restarts, and run-time changes will override settings in
+      <filename>clients.conf</filename>.  However, if a setting is
+      <emphasis>changed</emphasis> (or a client added, or removed) in
+      <filename>clients.conf</filename>, this will take precedence.
+    </para>
+  </refsect1>
+  
   <refsect1 id="dbus_interface">
     <title>D-BUS INTERFACE</title>
     <para>

=== modified file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2011-12-27 03:56:39 +0000
+++ network-hooks.d/bridge	2012-04-24 06:55:34 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -15,13 +18,18 @@
 
 CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
 
-# Read config file, which must set "BRIDGE", "PORTS", and optionally
-# "IPADDRS" and "ROUTES".
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,' -e "/^${BRIDGE}\$/d"
+}
+
+# Read config file, which must set "BRIDGE", "PORT_ADDRESSES", and
+# optionally "IPADDRS" and "ROUTES".
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
 fi
 
-if [ -z "$BRIDGE" -o -z "$PORTS" ]; then
+if [ -z "$BRIDGE" -o -z "$PORT_ADDRESSES" ]; then
     exit
 fi
 
@@ -29,40 +37,48 @@
     exit
 fi
 
-for b in /sbin/brctl /usr/sbin/brctl; do
+brctl="/sbin/brctl"
+for b in "$brctl" /usr/sbin/brctl; do
     if [ -e "$b" ]; then
 	brctl="$b"
 	break
     fi
 done
 
-case "$1" in
-    start)
-	"$brctl" addbr "$BRIDGE"
-	for port in $PORTS; do
-	    "$brctl" addif "$BRIDGE" "$port"
-	    ip link set dev "$port" up
-	done
-	ip link set dev "$BRIDGE" up
-	sleep "$DELAY"
-	if [ -n "$IPADDRS" ]; then
-            for ipaddr in $IPADDRS; do
-		ip addr add "$ipaddr" dev "$BRIDGE"
-	    done
-	fi
-	if [ -n "$ROUTES" ]; then
-            for route in $ROUTES; do
-		ip route add "$route" dev "$BRIDGE"
-	    done
-	fi
-	;;
-    stop)
-	ip link set dev "$BRIDGE" down
-	for port in $PORTS; do
-	    ip link set dev "$port" down
-	    "$brctl" delif "$BRIDGE" "$port"
-	done
-	"$brctl" delbr "$BRIDGE"
+do_start(){
+    "$brctl" addbr "$BRIDGE"
+    for address in $PORT_ADDRESSES; do
+	interface=`addrtoif "$address"`
+	"$brctl" addif "$BRIDGE" "$interface"
+	ip link set dev "$interface" up
+    done
+    ip link set dev "$BRIDGE" up
+    sleep "${DELAY%%.*}"
+    if [ -n "$IPADDRS" ]; then
+        for ipaddr in $IPADDRS; do
+	    ip addr add "$ipaddr" dev "$BRIDGE"
+	done
+    fi
+    if [ -n "$ROUTES" ]; then
+        for route in $ROUTES; do
+	    ip route add "$route" dev "$BRIDGE"
+	done
+    fi
+}
+
+do_stop(){
+    ip link set dev "$BRIDGE" down
+    for address in $PORT_ADDRESSES; do
+	interface=`addrtoif "$address"`
+	ip link set dev "$interface" down
+	"$brctl" delif "$BRIDGE" "$interface"
+    done
+    "$brctl" delbr "$BRIDGE"
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo /bin/ip

=== modified file 'network-hooks.d/bridge.conf'
--- network-hooks.d/bridge.conf	2011-11-24 20:15:24 +0000
+++ network-hooks.d/bridge.conf	2011-12-31 13:25:58 +0000
@@ -2,7 +2,7 @@
 
 #BRIDGE=br0
 
-#PORTS="eth0 eth1"
+#PORT_ADDRESSES="00:11:22:33:44:55 11:22:33:44:55:66"
 
 ## Optional
 

=== modified file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2011-12-02 16:52:50 +0000
+++ network-hooks.d/openvpn	2012-04-24 06:55:34 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -16,7 +19,9 @@
 CONFIG="openvpn.conf"
 
 # Extract the "dev" setting from the config file
-VPNDEVICE="`sed -n -e 's/#.*//' -e 's/^[[:space:]]*dev[[:space:]]\+//p' \"$MANDOSNETHOOKDIR/$CONFIG\"`"
+VPNDEVICE=`sed -n -e 's/[[:space:]]#.*//' \
+    -e 's/^[[:space:]]*dev[[:space:]]\+//p' \
+    "$MANDOSNETHOOKDIR/$CONFIG"`
 
 PIDFILE=/run/openvpn-mandos.pid
 
@@ -32,16 +37,22 @@
 
 openvpn=/usr/sbin/openvpn
 
-case "$1" in
-    start)
-	"$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" --config "$CONFIG"
-	sleep "$DELAY"
-	;;
-    stop)
-	PID="`cat \"$PIDFILE\"`"
-	if [ "$PID" -gt 0 ]; then
-	    kill "$PID"
-	fi
+do_start(){
+    "$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' \
+	--writepid "$PIDFILE" --config "$CONFIG"
+     sleep "$DELAY"
+}
+
+do_stop(){
+    PID="`cat \"$PIDFILE\"`"
+    if [ "$PID" -gt 0 ]; then
+	kill "$PID"
+    fi
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo "$openvpn"

=== modified file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2011-12-30 18:29:33 +0000
+++ network-hooks.d/wireless	2012-04-24 06:55:34 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -20,6 +23,11 @@
 
 CONFIG="$MANDOSNETHOOKDIR/wireless.conf"
 
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+}
+
 # Read config file
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
@@ -27,13 +35,16 @@
     exit
 fi
 
-interfaces="`env|sed -n -e 's/^\(MODULE\|IPADDRS\|ROUTES\|WPA_DRIVER\)_\([^=]*\)=.*/\2/p' \"$CONFIG\" |sort -u`"
+ifkeys=`env | sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' "$CONFIG" \
+    | sort -u`
 
 # Exit if DEVICE is set and is not any of the wireless interfaces
 if [ -n "$DEVICE" ]; then
     while :; do
-	for IF in $interfaces; do
-	    if [ "$IF" = "$DEVICE" ]; then
+	for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
+	    if [ "$INTERFACE" = "$DEVICE" ]; then
 		break 2
 	    fi
 	done
@@ -62,64 +73,81 @@
     WPAS_OPTIONS="-P$PIDFILE $WPAS_OPTIONS"
 fi
 
-case "${MODE:-$1}" in
-    start)
-	mkdir -m u=rwx,go= -p "$CTRLDIR"
-	"$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
-        for INTERFACE in $interfaces; do
-	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$INTERFACE"\"`
-	    IFDELAY=`eval 'echo "$DELAY_'"$INTERFACE"\"`
-	    "$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
-		"${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
-		| sed -e '/^OK$/d'
-	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
-	    eval wpa_interface_"$INTERFACE"
-	    "$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
-		"$NETWORK" | sed -e '/^OK$/d'
-	    sleep "${IFDELAY:-$DELAY}" &
-	    sleep=$!
-	    while :; do
-		kill -0 $sleep 2>/dev/null || break
-		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status | sed -n -e 's/^wpa_state=//p'`
-		if [ "$STATE" = COMPLETED ]; then
-		    kill $sleep 2>/dev/null
-		    break
-		fi
-		sleep 1
-	    done &
-	    wait $sleep || :
-	    IPADDRS=`eval 'echo "$IPADDRS_'"$INTERFACE"\"`
-	    if [ -n "$IPADDRS" ]; then
-		if [ "$IPADDRS" = dhcp ]; then
-		    ipconfig -c dhcp -d "$INTERFACE" || :
-		    #dhclient "$INTERFACE"
-		else
-		    for ipaddr in $IPADDRS; do
-			"$ip" addr add "$ipaddr" dev "$INTERFACE"
-		    done
-		fi
-	    fi
-	    ROUTES=`eval 'echo "$ROUTES_'"$INTERFACE"\"`
-	    if [ -n "$ROUTES" ]; then
-		for route in $ROUTES; do
-		    "$ip" route add "$route" dev "$BRIDGE"
-		done
-	    fi
-        done
-	;;
-    stop)
-	"$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
-        for INTERFACE in $interfaces; do
-	    "$ip" addr show scope global permanent dev "$INTERFACE" \
-		| while read type addr rest; do
+do_start(){
+    mkdir -m u=rwx,go= -p "$CTRLDIR"
+    "$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
+    for KEY in $ifkeys; do
+	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	INTERFACE=`addrtoif "$ADDRESS"`
+	DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
+	IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
+	"$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
+	    "${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
+	    | sed -e '/^OK$/d'
+        NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
+	eval wpa_interface_"$KEY"
+	"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
+	    "$NETWORK" | sed -e '/^OK$/d'
+	sleep "${IFDELAY:-$DELAY}" &
+	sleep=$!
+	while :; do
+	    kill -0 $sleep 2>/dev/null || break
+	    STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status \
+		| sed -n -e 's/^wpa_state=//p'`
+	    if [ "$STATE" = COMPLETED ]; then
+		while :; do
+		    kill -0 $sleep 2>/dev/null || break 2
+		    UP=`cat /sys/class/net/"$INTERFACE"/operstate`
+		    if [ "$UP" = up ]; then
+			kill $sleep 2>/dev/null
+			break 2
+		    fi
+		    sleep 1
+		done
+	    fi
+	    sleep 1
+	done &
+	wait $sleep || :
+	IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
+	if [ -n "$IPADDRS" ]; then
+	    if [ "$IPADDRS" = dhcp ]; then
+		ipconfig -c dhcp -d "$INTERFACE" || :
+		#dhclient "$INTERFACE"
+	    else
+		for ipaddr in $IPADDRS; do
+		    "$ip" addr add "$ipaddr" dev "$INTERFACE"
+		done
+	    fi
+	fi
+	ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
+	if [ -n "$ROUTES" ]; then
+	    for route in $ROUTES; do
+		"$ip" route add "$route" dev "$BRIDGE"
+	    done
+	fi
+    done
+}
+
+do_stop(){
+    "$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
+    for KEY in $ifkeys; do
+	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	INTERFACE=`addrtoif "$ADDRESS"`
+	"$ip" addr show scope global permanent dev "$INTERFACE" \
+	    | while read type addr rest; do
 		case "$type" in
 		    inet|inet6)
 			"$ip" addr del "$addr" dev "$INTERFACE"
 			;;
 		esac
 	    done
-	    "$ip" link set dev "$INTERFACE" down
-	done
+	"$ip" link set dev "$INTERFACE" down
+    done
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo "$wpa_supplicant"

=== modified file 'network-hooks.d/wireless.conf'
--- network-hooks.d/wireless.conf	2011-12-30 18:26:04 +0000
+++ network-hooks.d/wireless.conf	2011-12-31 13:25:58 +0000
@@ -2,9 +2,10 @@
 #WPAS_OPTIONS=""
 
 # wlan0
-MODULE_wlan0=ath9k
-#WPA_DRIVER_wlan0=wext
-wpa_interface_wlan0(){
+ADDRESS_0=00:11:22:33:44:55
+MODULE_0=ath9k
+#WPA_DRIVER_0=wext
+wpa_interface_0(){
     # Use this format to set simple things:
     wpa_cli_set ssid home
     wpa_cli_set psk "secret passphrase"
@@ -12,7 +13,11 @@
     #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" bssid "$NETWORK" 00:11:22:33:44:55
     #"$wpa_cli" -g "$CTRL" ping
 }
-#DELAY_wlan0=10
-IPADDRS_wlan0=dhcp
-#IPADDRS_wlan0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
-#ROUTES_wlan0="192.0.2.0/24 2001:DB8::/32"
+#DELAY_0=10
+IPADDRS_0=dhcp
+#IPADDRS_0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
+#ROUTES_0="192.0.2.0/24 2001:DB8::/32"
+
+#ADDRESS_1=11:22:33:44:55:66
+#MODULE_1=...
+#...

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2011-12-25 00:40:09 +0000
+++ plugin-runner.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -426,7 +426,7 @@
     errno = 0;
     switch(key){
       char *tmp;
-      intmax_t tmpmax;
+      intmax_t tmp_id;
     case 'g': 			/* --global-options */
       {
 	char *plugin_option;
@@ -505,24 +505,24 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (uid_t)tmpmax){
+	 or tmp_id != (uid_t)tmp_id){
 	argp_error(state, "Bad user ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)uid);
 	break;
       }
-      uid = (uid_t)tmpmax;
+      uid = (uid_t)tmp_id;
       break;
     case 131:			/* --groupid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (gid_t)tmpmax){
+	 or tmp_id != (gid_t)tmp_id){
 	argp_error(state, "Bad group ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)gid);
 	break;
       }
-      gid = (gid_t)tmpmax;
+      gid = (gid_t)tmp_id;
       break;
     case 132:			/* --debug */
       debug = true;

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2011-10-05 16:56:06 +0000
+++ plugin-runner.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2011-10-05">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2011-12-24 23:17:02 +0000
+++ plugins.d/askpass-fifo.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Askpass-FIFO - Read a password from a FIFO and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/askpass-fifo.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "askpass-fifo">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-12-30 17:14:15 +0000
+++ plugins.d/mandos-client.c	2011-12-31 23:05:34 +0000
@@ -9,8 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -1474,7 +1474,7 @@
 	}
 	free(delaystring);
 	if(connect_to != NULL){
-	  ret = setenv("CONNECT", connect_to);
+	  ret = setenv("CONNECT", connect_to, 1);
 	  if(ret == -1){
 	    perror_plus("setenv");
 	    _exit(EX_OSERR);

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2011-12-27 03:56:39 +0000
+++ plugins.d/mandos-client.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2011-11-27">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2011-12-25 00:40:09 +0000
+++ plugins.d/password-prompt.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Password-prompt - Read a password from the terminal and print it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -124,13 +124,13 @@
     int ret;
     int cl_fd;
     {
-      uintmax_t maxvalue;
+      uintmax_t proc_id;
       char *tmp;
       errno = 0;
-      maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+      proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
       
       if(errno != 0 or *tmp != '\0'
-	 or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+	 or proc_id != (uintmax_t)((pid_t)proc_id)){
 	return 0;
       }
     }
@@ -514,6 +514,7 @@
 	switch(e){
 	case EBADF:
 	  status = EX_UNAVAILABLE;
+	  break;
 	case EIO:
 	case EINVAL:
 	default:

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/password-prompt.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/plymouth.c'
--- plugins.d/plymouth.c	2011-12-25 00:40:09 +0000
+++ plugins.d/plymouth.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Plymouth - Read a password from Plymouth and output it
  * 
- * Copyright © 2010-2011 Teddy Hogeborn
- * Copyright © 2010-2011 Björn Påhlsson
+ * Copyright © 2010-2012 Teddy Hogeborn
+ * Copyright © 2010-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -218,13 +218,13 @@
 int is_plymouth(const struct dirent *proc_entry){
   int ret;
   {
-    uintmax_t maxvalue;
+    uintmax_t proc_id;
     char *tmp;
     errno = 0;
-    maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+    proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
 
     if(errno != 0 or *tmp != '\0'
-       or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+       or proc_id != (uintmax_t)((pid_t)proc_id)){
       return 0;
     }
   }
@@ -265,36 +265,36 @@
 
 pid_t get_pid(void){
   int ret;
-  uintmax_t maxvalue = 0;
+  uintmax_t proc_id = 0;
   FILE *pidfile = fopen(plymouth_pid, "r");
   /* Try the new pid file location */
   if(pidfile != NULL){
-    ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+    ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
     if(ret != 1){
-      maxvalue = 0;
+      proc_id = 0;
     }
     fclose(pidfile);
   }
   /* Try the old pid file location */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     pidfile = fopen(plymouth_pid, "r");
     if(pidfile != NULL){
-      ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+      ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
       if(ret != 1){
-	maxvalue = 0;
+	proc_id = 0;
       }
       fclose(pidfile);
     }
   }
   /* Look for a plymouth process */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     struct dirent **direntries = NULL;
     ret = scandir("/proc", &direntries, is_plymouth, alphasort);
     if (ret == -1){
       error_plus(0, errno, "scandir");
     }
     if (ret > 0){
-      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &maxvalue);
+      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &proc_id);
       if (ret < 0){
 	error_plus(0, errno, "sscanf");
       }
@@ -304,8 +304,8 @@
     free(direntries);
   }
   pid_t pid;
-  pid = (pid_t)maxvalue;
-  if((uintmax_t)pid == maxvalue){
+  pid = (pid_t)proc_id;
+  if((uintmax_t)pid == proc_id){
     return pid;
   }
   

=== modified file 'plugins.d/plymouth.xml'
--- plugins.d/plymouth.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/plymouth.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plymouth">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2011-12-24 23:17:02 +0000
+++ plugins.d/splashy.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Splashy - Read a password from splashy and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/splashy.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "splashy">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2011-12-24 23:17:02 +0000
+++ plugins.d/usplash.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Usplash - Read a password from usplash and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/usplash.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "usplash">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

